;=================================================================
; CUSTOM STRUCTURE DEFINITIONS AND MACROS
;=================================================================
; These are struc-type data structure that are extended
; to use register addressing and other macro extensions.
; Many thanks to Tomasz Grysztar for the help with the structure definitions. Also,
; many of the example macros from FASM.PDF v1.64 documentation have been
; re-used here; I will take no credit for those.

;=================================================================
; MACROS
;=================================================================
; These are the macros used to define the custom structures more efficiently, even
; though the new macros themselves may not be efficiently written.

; list of custom strucs recorded by macros

customstruclist   equ

; adds struc name to list of names

macro addcustomstructolist strucname {
      match any, customstruclist \{ customstruclist equ customstruclist,strucname \}
      match , customstruclist \{ customstruclist equ strucname \}
      }

; defines new structure with name and sets labels for later
; (sizeof., virtual instance, member labels)

macro rcustomstruc strucname {
      addcustomstructolist strucname				; adds struc name to struc list (for later)
      struc strucname { 					; start "struc"
       label .#strucname#_TEMP_LABEL
       laststrucnamestartlabel equ .#strucname#_TEMP_LABEL
      }

customstruc	fix    rcustomstruc				; (just renames macro)
endcs		fix    restore laststrucnamestartlabel }	; restore is essential for nested structures

; offsets next struc member by certain amount of bytes from struc
; start by filling struc with anonymous bytes

macro member moffset,labelname,labeltypeext {
      times (moffset-($-laststrucnamestartlabel)) db ?
      labelname       labeltypeext
      }

; used to ensure struc size attains known size if last members are unknown

macro strucsize moffset {
      times (moffset-($-laststrucnamestartlabel)) db ?
      }

; creates the "virtual" data used to address the custom strucs by register index in-memory

macro createcustomstrucvirtual struclist {
      match params,struclist \{ createcustomstrucvirtualsub params \}
      }

; helper macro for createcustomstrucvirtualsub macro
; (creates "virtual at 0" instance of structure and defines sizeof. constant)

macro createcustomstrucvirtualsub [strucname] {
      local sizelabel1
      virtual at 0
	   sizelabel1:
	   strucname strucname
	   sizeof.#strucname = ($-sizelabel1)			; defines struc size as you would with a "struct" definition
      end virtual
      }

;=================================================================
; CUSTOM STRUCTURES
;=================================================================
; "member" instruction is used to offset the struc member on the following
; line by the specified amount of bytes from the start of the structure.
; It must be used in order and offsets must be equal to or greater than accumulated
; definition size from after last member, or will not compile (will give "invalid value" error).
; It is not mandatory.
; "strucsize" has the same behaviour, but is used at the end of
; structures to ensure the structure has a certain length (sizeof.[strucname]),
; which is useful if the last member(s) of the structure is/are not defined
; or is/are unknown. All structure sizes can be accessed via the "sizeof.strucname"
; instruction usually defined by the "struct" macro.



