;=======================================================
; checks level info before modifying
;=======================================================
; CALLED BEFORE EVERY LEVEL START/AFTER EVERY LEVEL LOAD
; (called after calls near "Finished Loading!" string reference)
; NOTE: CStartTriggers are played BEFORE this code.
;
; This is called following
; the main game "level load" call (which also makes reference
; to "Finished Loading!" string). This call is used for ALL
; level loads, thus including restarts and savegames. It should
; take place after CStartTriggers. If a breakpoint is placed here, it will typically
; halt loading with the progress bar at 100%.

; DO NOT RELY ON STACK - ALREADY NEAR LIMIT

;=======================================================
INTERCEPTPROC POSTLEVELLOADINTERCEPT
;=======================================================

add esp,(604h+4)			; REPLACED CODE (modified to include return address)
push DWORD [esp-(604h+4)]		; move return address

; NEW IN ATX 1.61 - TEST FOR LEVEL LOAD SUCCESS PRIOR TO ATTEMPTING POST LEVEL LOAD CODE

test eax,eax				; could not load properly if result from last call was above zero
jge @f

	ret

@@:

push ebx
push ecx
push edx
push esi
push edi
push eax

	call LOADINITIALBITMAP

;----------------------------------------------
; Actions to perform at every level load (before all other function initializations)
;----------------------------------------------

	cmp [NewGRFLoadDetected],FALSE
	    jz @f

		mov [LINSTEnabled],FALSE
		mov [CAnimalDisplayIndex],0
		call CreateMainCAnimalBuffer

	@@:

;----------------------------------------------
; Actions to perform at every level load ONLY if the level is not a saved game
; and level restart is important
;----------------------------------------------

	call IsLevelSavedGame
	test eax,eax
	     jnz @f

		call SetGameDifficulty
		call ModifyCAnimalHP
		call SetRandomInstanceStartPositions

	@@:

;----------------------------------------------
; Actions to perform only at new GRF loads (that may be savegames)
;----------------------------------------------

	cmp [NewGRFLoadDetected],FALSE
	    jz @f

		call ModifyCAnimalSleepWakeUp

	@@:

;----------------------------------------------
; Actions to perform at every level load
;----------------------------------------------

	call SaveCAnimalInitialData			; must be after instance randomizer call
	call DoAT10JUMPMainEmulInit
	call DoAnimalProjectileInit

	call DoNightModeTeleportsInit

; PRINT CANIMAL COUNT STRING FOR CANIMAL CHEAT

	call GetCAnimalInstanceCount
	cinvoke wsprintf,CAnimalTitleBuffer,CAnimalTitleBuffer_base,eax
	mov [CAnimalTitleSize],eax

; RESET PLAYER'S AITYPE AND MANAGE AT113

	call FLY_CLOAK_Cheats_Reset

	mov [AT113PlayerAITypeNon5Set],FALSE		; set to true if originally enabled during savegame creation
	cmp [AT113SaveGameAITypeUsed],TRUE
	    jnz @f

		mov [AT113PlayerAITypeNon5Set],TRUE
		push [AT113SaveGameAITypeValue]
		call ChangePlayerPrivAIType

	@@:

; SET IN-GAME KEYS AND MOUSE MOVEMENT AND MANAGE AT112

	call EnableInGameKeys
	mov [DisableInGameMouseMvt],FALSE
	mov ecx,(at112inputtogglerid-8) 		; (we know how AT112 works, so a little hack can't hurt...)
	call AT112MAIN

; CPARTICLES CODE

	call CParticlesEndLevelLoad			; MUST BE BEFORE Restart_Exit_Level_BOOL] reset

; SKY AND FOG MODIFICATIONS

	call SetSkyFogColorNearFarInit			; (used for when game does not summon CStartTriggers)

; SET FIRST FRAME AS COUNTER START (TIME CHEAT)

	mov [CounterFirstRun],1

; ENSURE SOME CHEAT DISABLED

      ;  mov [ATXDRV_ENABLED],FALSE                      ; NOT IMPLEMENTED
	xor al,al
	or al,[FDEnabled]
	or al,[THIRDPEnabled]
	mov [FDEnabled],FALSE
	mov [THIRDPEnabled],FALSE
	mov [THIRDPKeyAimToggleOn],FALSE
	mov [OBJNInstance],NULL
	mov [UseObjCheat],FALSE
	test al,al
	     jz @f

		call ResetCCameraPlayerSources

	@@:

;----------------------------------------------
; Actions to perform after all others at every level start
;----------------------------------------------

       ; BBMCALL RainEffectNewLoadInit,[RainFeatureEnabled]

	cmp [AT118EnableFastLoadNoSave],0
	    jz .at118nofastload

		LOCALRELEASE [AT118InstanceRefBuf]
		mov [AT118BufDesc.InstanceCount],0
		mov [AT118BufDesc.InstRefBufCut],0
		jmp .at118hasfastloadnosave

	.at118nofastload:

	call AT118InitInstRefBuffer

	.at118hasfastloadnosave:

	call InitPlayerPathTraces

	call AdjustCameraFOVWidescreen
	mov [THIRDPAIMAimdotLastObj],0

	mov [Restart_Exit_Level_BOOL],0

       ; call StartInGameMenuTimer      ; obsolete in ATX 2.06
	call SetInitialLevelTickCount

	mov [QuickSaveTick],0
	mov [QuickSaveSaveDone],0
	mov [TempMessageTime],0

	call FIXOBJECTSPEED_INIT			; must be after DoAnimalProjectileInit call

	xor eax,eax
	mov al,[NewGRFLoadDetected]
	PLUGINSCBINVOKE PLUGIN_CALLBACKS.PostLevelLoad,eax

pop eax
pop edi
pop esi
pop edx
pop ecx
pop ebx
ret			; (this marks the end of ATX features used during level load)

