;============================================================
ATXINITPROC FindLoadPlugins
;============================================================
; loads pluginqueue with found plugins (if any)

push esi
push edi

; init vars

	mov [plugincbcounttotal],0

	mov eax,-1
	mov edx,sizeof.PLUGIN_DESC
	mov ecx,pluginqueue
	call QUEUE_Reset			; initialize plugin queue

; print plugin scan start to logfile

	stdcall WriteLogFileInGame,PLUGIN_LOGFILE_MSG_BEGIN,PLUGIN_LOGFILE_MSG_BEGIN.size

; get base plugin folder path + search string

	mov esi,mapsdirpath
	mov edi,plugincurrent.FolderPath
	mov ecx,[mapsdirpathlength]
	rep movsb
	mov esi,PLUGIN_MAPS_FOLDER_NAME
	call TransferString

	mov BYTE [edi],'\'			; include null-terminator
	inc edi
	mov esi,PLUGIN_FOLDER_SEARCH_STR
	mov [plugintempfolderpathend],edi
	call TransferString

; copy base game dir

	mov esi,gamedirpath
	mov edi,plugincurrent.DLLPath
	mov ecx,[gamedirpathlength]
	rep movsb
	mov [plugintempdllpathend],edi

; find first folder

	invoke FindFirstFile,plugincurrent.FolderPath,plugincurrentfd
	cmp eax,INVALID_HANDLE_VALUE
	    jz .exit

	mov ebx,eax

; loop for next folders

	.checkcurrentfoundfile:

	mov eax,[plugincurrentfd.dwFileAttributes]
	test eax,FILE_ATTRIBUTE_HIDDEN+FILE_ATTRIBUTE_SYSTEM+FILE_ATTRIBUTE_TEMPORARY
	     jnz .searchfornextfolder

	test eax,FILE_ATTRIBUTE_DIRECTORY
	     jz .searchfornextfolder

	cmp BYTE [plugincurrentfd.cFileName],'.'
	    jz .searchfornextfolder

	; set plugin's folder path

		mov esi,plugincurrentfd.cFileName
		mov edi,[plugintempfolderpathend]
		call TransferString
		mov WORD [edi],'\'		; include null-terminator

	; set plugin's DLL name path

		mov esi,plugincurrentfd.cFileName
		mov edi,[plugintempdllpathend]
		call TransferString
		mov BYTE [edi],'.'
		mov esi,EXTENSION_DLL_STR
		inc edi
		call TransferString

	; attempt to load DLL with plugin's DLL name path

	; print to logfile

		cinvoke wsprintf,plugintemplogprint,PLUGIN_LOGFILE_MSG_FOLDERFOUND,\
				 plugincurrent.DLLPath,plugincurrent.FolderPath
		stdcall WriteLogFileInGame,plugintemplogprint,eax

		invoke LoadLibrary,plugincurrent.DLLPath
		test eax,eax
		     jz .pluginloadfail

		; print success to logfile

			stdcall WriteLogFileInGame,PLUGIN_LOGFILE_MSG_LOADSUCCESS,PLUGIN_LOGFILE_MSG_LOADSUCCESS.size

		; find plugin's DLL callbacks

			mov [plugincurrent.Handle],eax
			mov esi,plugincallbacknames
			mov edi,plugincurrent.Callbacks
			mov [plugincbcountcurrent],0

			.plugincallbacksloop:
			invoke GetProcAddress,[plugincurrent.Handle],[esi]
			test eax,eax
			mov [edi],eax			; null if not implemented by plugin (must be initialized)
			     jz .callbacknotfound

			; print to logfile

				cinvoke wsprintf,plugintemplogprint,PLUGIN_LOGFILE_MSG_LOADCALLBACK,[esi]
				stdcall WriteLogFileInGame,plugintemplogprint,eax
				inc [plugincbcountcurrent]
				inc [plugincbcounttotal]

			.callbacknotfound:
			add esi,4
			add edi,4
			cmp esi,(plugincallbacknames+sizeof.PLUGIN_CALLBACKS)
			    jb .plugincallbacksloop

		; print callback count to logfile

			cinvoke wsprintf,plugintemplogprint,PLUGIN_LOGFILE_MSG_LOADCALLBACKCOUNT,[plugincbcountcurrent]
			stdcall WriteLogFileInGame,plugintemplogprint,eax

		; add plugin to official list

			mov edx,plugincurrent
			mov ecx,pluginqueue
			call QUEUE_Add_Always
			jmp .pluginloadexit

		.pluginloadfail:

	; print failure to logfile
		stdcall WriteLogFileInGame,PLUGIN_LOGFILE_MSG_LOADFAIL,PLUGIN_LOGFILE_MSG_LOADFAIL.size
		.pluginloadexit:

; find next plugin folder

	.searchfornextfolder:

	invoke FindNextFile,ebx,plugincurrentfd
	test eax,eax
	     jnz .checkcurrentfoundfile

	invoke FindClose,ebx

.exit:

; print total plugin and callback counts

	cinvoke wsprintf,plugintemplogprint,PLUGIN_LOGFILE_MSG_END,[pluginqueue.nodecount],[plugincbcounttotal]
	stdcall WriteLogFileInGame,plugintemplogprint,eax

pop edi
pop esi
ret

;============================================================
ATXPROC FreeAllPlugins
;============================================================
cmp [pluginqueue.nodecount],0
    jz .noplugins

push esi
	mov esi,[pluginqueue.oldestnode]

	.freeloop:		; unload all DLLs
	invoke FreeLibrary,[esi+sizeof.STDQUEUENODE+PLUGIN_DESC.Handle]
	mov esi,[esi+STDQUEUENODE.newernode]
	test esi,esi
	     jnz .freeloop

	xor eax,eax
	xor edx,edx
	mov ecx,pluginqueue
	call QUEUE_Reset	; free queue

pop esi

.noplugins:
ret