;=====================================================
; TRESSTRDBG (WINDOWED) MAIN REAL-TIME DISPLAY CODE
;=====================================================
; [ebp-4] = DC for current screen writes
; [ebp-8] = DDRAW SURFACE OBJECT (0 if GUI)
; [ebp-12] = Window handle
; [ebp-16] = possible dump file handle
; [ebp-20] = temporary var
; [ebp-24] = start of parameters address
; [ebp-28] = string address
; [ebp-32] = converted string length

;=================================================
INTERCEPTPROC TRESSTRDBGINTERCEPT
;=================================================
push ebp
mov ebp,esp
sub esp,150
push edi
push esi
push ebx

	invoke FindWindow,TRES_WINDOW_CLASS_NAME,TRES_WINDOW_NAME
	mov [ebp-12],eax				; if window doesn't exist, string is
							; displayed on desktop
	invoke SetForegroundWindow,eax			; sets focus

; test parameter validity

	lea eax,[ebp+12]				; 2nd param
	mov edx,[ebp+8] 				; 1rst param (to test)
	cmp edx,[DATASEC_STARTADDR]
	    jb @f

		cmp edx,[DATASEC_ENDADDR]
		    jb .paramvalidated

	@@:

	lea eax,[ebp+16]				; 3rd param
	mov edx,[ebp+12]				; 2nd param (to test)
	cmp edx,[DATASEC_STARTADDR]
	    jb .exit

	cmp edx,[DATASEC_ENDADDR]
	    jnb .exit

	.paramvalidated:

	mov [ebp-24],eax
	mov [ebp-28],edx

	or ecx,0FFFFFFFFh

	@@:						; get string length in ecx (before sprintf)
	inc ecx
	cmp BYTE [edx+ecx],0
	    jnz @b

	cmp ecx,0
	    jz .exit
	cmp ecx,80					; accept string length of 80 bytes max
	    ja .exit					; its size can grow with sprintf = use caution

; get device context

	invoke GetDC,[ebp-12]
	test eax,eax
	mov [ebp-4],eax
	     jz .exit

	invoke SetTextColor,eax,0

; main TRESSTRDBG code (print to screen)

	sub esp,48					; max 48 bytes of parameters (assumed)

	push edi
	push esi
	mov esi,[ebp-24]				; start of parameters address
	lea edi,[esp+8] 				; to local space
	mov ecx,12
	rep movsd					; copy parameters passed to this function even if none
	pop esi 					; (offsets the cycles that would be used to check for parameters)

	mov ecx,25
	lea edi,[ebp-150+16]				; put space chars to main string buffer
	mov eax,20202020h				; ->needed to make sure strings are clear on-screen
	rep stosd
	pop edi

	lea edx,[ebp-150+16]				; destination for converted string (right after ADDRESS)
	invoke sprintf,edx,DWORD [ebp-28]		; convert string w/parameters
	add esp,(48+8)					; sprintf does not pop parameters

	lea edx,[ebp-150+16]
	or ecx,0FFFFFFFFh

	.countloop1:					; counts final string length and checks for bad
	inc ecx 					; chars (and decides finally if string is valid)
	mov al,[edx+ecx]
	cmp al,0Ah					; annoying char that pops up in many strings
	    jnz @f

	mov BYTE [edx+ecx],20h
	jmp .countloop1

	@@:
	test al,al
	     jnz .countloop1

	mov [ebp-32],ecx				; save string length (converted string returned from sprintf)
	mov BYTE [ecx+edx],' '				; put space over null-terminator

	invoke TextOut,DWORD [ebp-4],0,0,edx,100	; print main string w/100-byte length

	mov DWORD [ebp-150],078300A0Dh			; carriage return + '0x'
	mov DWORD [ebp-150+4], '0000'
	mov DWORD [ebp-150+8], '0000'
	mov DWORD [ebp-150+12], 0A0D2020h		; 2 spaces+another return

	lea edx,[ebp-150+11]				; last '0'
	mov eax,[ebp+4] 				; return address (to caller)

	.convertaddressloop:				; converts DWORD return address to string (hex)
	mov cl,al
	and cl,0Fh
	cmp cl,09h
	    jna @f
	add cl,7					; for letters
	@@:
	add [edx],cl
	shr eax,4
	dec edx
	cmp BYTE [edx],'x'
	    jnz .convertaddressloop

	lea edx,[ebp-150+2]
	invoke TextOut,DWORD [ebp-4],0,10h,edx,12	; print the ADDRESS ("RET:")

; Exit code

	invoke ReleaseDC,[ebp-12],[ebp-4]

	push [UseTresStrDbg]
	call SDBGWaitForInterval

.exit:
pop ebx
pop esi
pop edi
mov esp,ebp
pop ebp
ret

;=================================================
ATXPROC SDBGWaitForInterval			       ; parameter = interval (wait time)
;=================================================

push esi

	invoke GetTickCountTRUE
	mov esi,eax

	.waitloop:

	invoke GetAsyncKeyState,EMEREXITKEY
	test eax,80000000h				; high-order bit set if key pressed
	     jnz .exitkeydown

	invoke GetAsyncKeyState,RUNTHROUGHKEY
	test eax,80000000h
	     jnz .runkeydown

	invoke GetAsyncKeyState,PAUSEKEY
	test eax,80000000h
	     jnz .pausekeydown

	invoke GetTickCountTRUE
	sub eax,esi
	cmp eax,[esp+8]
	    jb .waitloop

pop esi
ret 4

.pausekeydown:						; pauses while F5 is pressed

	push edi
	xor edi,edi

	.pausekeydownloop:

	test edi,edi
	     jnz @f

		invoke GetAsyncKeyState,SAVETOTEXTKEY	; is write to file key down?
		test eax,80000000h
		     jz @f

		mov edi,eax
		call SDBGWriteToFile

	@@:

	invoke GetAsyncKeyState,PAUSEKEY
	test eax,80000000h
	     jnz .pausekeydownloop			; wait until F5 comes back up

pop edi
pop esi
ret 4

.runkeydown:						; exits loop quickly, can make dump

	invoke GetAsyncKeyState,SAVETOTEXTKEY
	test eax,80000000h
	     jz @f

		call SDBGWriteToFile

	@@:

pop esi
ret 4

.exitkeydown:

pop esi
add esp,4
pop ebx
pop esi
pop edi
mov esp,ebp
pop ebp
invoke ExitProcess,0					; force exit if F7 pressed
ret

;=================================================
ATXPROC SDBGWriteToFile
;=================================================
; Write data to file

	invoke CreateFile,SDBGDUMPFILE,GENERIC_WRITE,0,0,OPEN_ALWAYS,\
			  FILE_ATTRIBUTE_NORMAL,0

	mov [ebp-16],eax				; file handle
	invoke SetFilePointer,eax,0,0,FILE_END

	mov ecx,[ebp-32]				; string length
	lea edx,[ebp-150+16]				; string start
	mov WORD [edx+ecx],0A0Dh			; place ending carriage return

	add ecx,18					; 16 for retaddr, 2 for end carriage ret
							; ecx=number of bytes to write
	lea eax,[ebp-150]				; bufferstart (ret addr)
	lea edx,[ebp-20]				; #bytesread (unused)

	invoke WriteFile,[ebp-16],eax,ecx,edx,0
	invoke CloseHandle,[ebp-16]

ret
