;===========================================================
ATXPROC DrawPlayerPathTraces
;===========================================================
; PATH TRACES DRAWING/TIMING

push ebp
mov ebp,esp
sub esp,16
push esi
push ebx

; edi = screen DC

	mov esi,[PTPositions.oldestnode]
	test esi,esi
	     jz .exit

	invoke LoadBitmap,[dllhandle],102
	mov [ebp-4],eax
	invoke CreateCompatibleDC,edi
	mov [ebp-8],eax
	invoke SelectObject,eax,DWORD [ebp-4]

	.displayloop:

	fld DWORD [esi+sizeof.STDQUEUENODE+PTPOSINFO.x]
	fsub [MAPAnchorXValue]				; substract top-corner X-anchor from Xposition
	fdiv [MAPScaleXValue]				; Xfloatdisplacement / (X float units/pixel) = # pixels (Xpos)
	fistp DWORD [ebp-16]				; convert to integer and store

	fld DWORD [esi+sizeof.STDQUEUENODE+PTPOSINFO.y]
	fsub [MAPAnchorYValue]				; substract top-corner Y-anchor from Xposition
	fdiv [MAPScaleYValue]				; Yfloatdisplacement / (Y float units/pixel) = # pixels (Ypos)
	fistp DWORD [ebp-12]				; convert to integer and store

	xor ecx,ecx
	mov eax,[ebp-16]
	mov edx,[ebp-12]
	cmp eax,[MAPBitmapWidth]
	    jnb .skipposition
	cmp edx,[MAPBitmapHeight]
	    jnb .skipposition
	cmp eax,ecx
	    jl .skipposition
	cmp edx,ecx
	    jl .skipposition

    ; position values within image

	add eax,(6-1)					; add: (map displacement in pixels) - (half of position trace size in pixels)
	add edx,(6-1)
	mov [ebp-16],eax
	mov [ebp-12],edx

    ; set color of temp bitmap

	mov ebx,[PTColor]

    ; CALCULATE COLOR BASED ON TIMING

	invoke GetTickCount
	sub eax,[esi+sizeof.STDQUEUENODE+PTPOSINFO.starttick]
	cmp eax,[PTFadeDelay]
	    jb @f					; not passed fade-away mark?

	mov ecx,[PTMSecsDisplayPerPos]
	mov edx,ecx
	sub ecx,[PTFadeDelay]				; duration of fade
	sub edx,eax					; number of msecs left until disappears completely

	mov [esp-8],edx 				; number of msecs left for display
	mov [esp-4],ecx 				; duration of face
	fild DWORD [esp-8]
	fidiv DWORD [esp-4]

	mov eax,ebx
	mov ecx,ebx
	mov edx,ebx
	and eax,0FFh
	and ecx,0FF00h
	and edx,0FF0000h
	shr ecx,8
	shr edx,16
	mov [esp-4],eax
	mov [esp-8],ecx
	mov [esp-12],edx

	fild DWORD [esp-4]
	fmul st0,st1
	fistp DWORD [esp-4]
	fild DWORD [esp-8]
	fmul st0,st1
	fistp DWORD [esp-8]
	fild DWORD [esp-12]
	fmulp st1,st0
	fistp DWORD [esp-12]

	mov ebx,[esp-4]
	mov eax,[esp-8]
	mov edx,[esp-12]
	shl eax,8
	shl edx,16
	or ebx,eax
	or ebx,edx

	@@:

	invoke SetPixelV,[ebp-8],0,0,0
	invoke SetPixelV,[ebp-8],0,1,ebx
	invoke SetPixelV,[ebp-8],0,2,0
	invoke SetPixelV,[ebp-8],1,0,ebx
	invoke SetPixelV,[ebp-8],1,1,ebx
	invoke SetPixelV,[ebp-8],1,2,ebx
	invoke SetPixelV,[ebp-8],2,0,0
	invoke SetPixelV,[ebp-8],2,1,ebx
	invoke SetPixelV,[ebp-8],2,2,0

	invoke BitBlt,edi,[ebp-16],[ebp-12],3,3,[ebp-8],0,0,SRCPAINT

	.skipposition:

	mov esi,[esi+STDQUEUENODE.newernode]
	test esi,esi
	     jnz .displayloop

	invoke DeleteDC,DWORD [ebp-8]			; delete compatible DC
	invoke DeleteObject,DWORD [ebp-4]

.exit:

pop ebx
pop esi
mov esp,ebp
pop ebp
ret

;===========================================================
ATXPROC InitPlayerPathTraces
;===========================================================

	cmp [PTEnabled],0
	    jz @f

	mov eax,[PTMaxPos]
	mov edx,sizeof.PTPOSINFO
	mov ecx,PTPositions
	call QUEUE_Reset

	mov [PTRunOnce],0

	@@:
ret

;===========================================================
ATXPROC ResetPlayerPathTraces
;===========================================================

	cmp [PTEnabled],0
	    jz @f

	xor eax,eax
	xor edx,edx
	mov ecx,PTPositions
	call QUEUE_Reset

	mov [PTRunOnce],0

	@@:
ret

;===========================================================
ATXPROC CalcPlayerPathTracesRT
;===========================================================

	cmp [PTRunOnce],FALSE
	    jnz .doproc

	invoke GetTickCount
	mov [PTLastTick],eax
	mov [PTRunOnce],TRUE

ret

.doproc:

push ebx
push esi

	invoke GetTickCount
	mov ebx,eax

	mov esi,[PTPositions.oldestnode]
	test esi,esi
	     jz .calcloopend

	.calcloop:

	mov ecx,ebx
	sub ecx,[esi+sizeof.STDQUEUENODE+PTPOSINFO.starttick]
	cmp ecx,[PTMSecsDisplayPerPos]
	    jb .calcloopend

	mov esi,[esi+STDQUEUENODE.newernode]		; get newest node from current before current is deleted
	mov ecx,PTPositions
	call QUEUE_DeleteLast

	test esi,esi
	     jnz .calcloop

	.calcloopend:

	mov eax,ebx
	sub eax,[PTLastTick]
	cmp eax,[PTInterval]
	    jb @f

	call CalcPlayerPathTracesRT_AddCurrentPos
	mov [PTLastTick],ebx

	@@:

pop esi
pop ebx
ret

;===========================================================
ATXPROC CalcPlayerPathTracesRT_AddCurrentPos
;===========================================================
; ebx = tick count

push esi

	GETPLAYERLOCADDR

	mov ecx,[eax]
	mov edx,[eax+4]
	mov [PT_tempvalues.x],ecx
	mov [PT_tempvalues.y],edx
	mov ecx,[eax+8]
	mov [PT_tempvalues.z],ecx
	mov [PT_tempvalues.starttick],ebx


; check if position already exists (via x-y)

	mov esi,[PTPositions.oldestnode]
	test esi,esi
	     jz .addloopend

	.addloop:

	fld DWORD [esi+sizeof.STDQUEUENODE+PTPOSINFO.x]

	fsub [PT_tempvalues.x]
	fmul st0,st0
	fld DWORD [esi+sizeof.STDQUEUENODE+PTPOSINFO.y]

	fsub [PT_tempvalues.y]
	fmul st0,st0
	faddp st1,st0

	fcomp [PTRadiusExp2]
	fstsw ax
	test ah,1
	     jnz .exit

	mov esi,[esi+STDQUEUENODE.newernode]
	test esi,esi
	     jnz .addloop

	.addloopend:

; add position

	mov ecx,PTPositions
	mov edx,PT_tempvalues
	call QUEUE_Add_Trunc

.exit:
pop esi
ret


