;========================================================
;MAIN FOG AND SKY MOD CODE/FUNCTIONS
;========================================================

;========================================================
INTERCEPTPROC AT05FOGMAINPROCSUBSTITUTE 			; ecx = AT05 object
;========================================================
; This saves every new fog color set by the level's script via
; AT05 and sets the new color

	push ecx
	push esi
	push edi
	lea esi,[ecx+4]
	mov edi,FogTempObj+4
	mov ecx,5
	rep movsd						; copy AT05 object without VTable (simple - no sub-classes)

; fog color check

	mov ecx,FogTempObj
	test BYTE [ecx+08],CSETFOG_USE_RGB			; are we setting new fog color???
	     jz .nofogcolor

	test [NIGHTEnabled],1
	     mov [AT05FogWithColorModRunOnceInLevel],TRUE
	     jz .nofogcolor

		mov eax,[ecx+0Ch]				; SAVE NEW FOG COLOR
		mov edx,[NIGHTFogColorCurrent]
		mov [FogColorSaved],eax
		mov [ecx+0Ch],edx				; set new fog color

	.nofogcolor:

; fogpower check

	test BYTE [ecx+08],CSETFOG_USE_FOGPOWER
	     jz .nofogpower

		mov eax,[ecx+CSetFogAction.FogPower]

		test [NIGHTEnabled],TRUE
		mov edx,[NIGHTFogPower]
		     jz @f
		cmp edx,-1.000000
		    jz @f

			mov eax,edx

		@@:

		test [DisableGameFog],TRUE
		     jz @f

			mov eax,[FogSettingsForcedFogPowerValue]

		@@:

		mov [ecx+CSetFogAction.FogPower],eax

	.nofogpower:

; foghalf check

	test BYTE [ecx+08],CSETFOG_USE_FOGHALF
	     jz .nofoghalf

		mov eax,[ecx+CSetFogAction.FogHalf]

		test [NIGHTEnabled],1
		mov edx,[NIGHTFogHalf]
		     jz .nofoghalf

		cmp edx,-1.000000
		    jz @f

			mov eax,edx

		@@:

		test [DisableGameFog],TRUE
		     jz @f

			mov eax,[FogSettingsForcedFogHalfValue]

		@@:

		mov [ecx+CSetFogAction.FogHalf],eax

	.nofoghalf:

	call [AT05FOGMAINPROCADDR]

	pop edi
	pop esi
	pop ecx

	mov edx,[FogTempObj+4]					; save reference count in real object (don't know if necessary)
	mov [ecx+4],edx

ret

;========================================================
INTERCEPTPROC AT13SKYMAINPROCSUBSTITUTE 			; ecx = AT13 object
;========================================================
; this prevents the level script from modifying any FogNear/FogFar/Height values

	push ecx
	push esi
	push edi
	lea esi,[ecx+4]
	mov edi,SkyTempObj+4
	mov ecx,9
	rep movsd						; copy AT13 object without VTable (simple - no sub-classes)

	test [NIGHTEnabled],1
	mov ecx,SkyTempObj
	     jz .nonightmode

		mov eax,[NIGHTFogNearCurrent]
		mov edx,[NIGHTFogFarCurrent]
		mov [ecx+14h],eax
		mov eax,[NIGHTSkyHeight]
		mov [ecx+18h],edx
		mov [ecx+10h],eax

	.nonightmode:

	call [AT13SKYMAINPROCADDR]

	pop edi
	pop esi
	pop ecx

	mov edx,[SkyTempObj+4]					; save reference count in real object (don't know if necessary)
	mov [ecx+4],edx

ret

;========================================================
ATXPROC SetSkyFogColorNearFarInit
;========================================================
push esi
push edi

;-------------
;SKY INIT
;-------------

cmp [NIGHTEnabled],FALSE
    jz .noskyinit

; set initial Sky FogNear/FogFar/Height settings (whether previously set or not)

	mov edi,SkyTempObj+4
	xor eax,eax
	mov ecx,9
	rep stosd

	mov ecx,SkyTempObj
	mov eax,[NIGHTFogNearCurrent]
	mov edx,[NIGHTFogFarCurrent]
	mov [ecx+14h],eax
	mov eax,[NIGHTSkyHeight]
	mov [ecx+18h],edx
	mov [ecx+10h],eax
	mov DWORD [ecx+8],2+4+8 				; these bits tell AT13 to only change FogNear/FogFar/Height
	call [AT13SKYMAINPROCADDR]


; set initial Ambient Light setting (if applicable)

	fld [NIGHTAmbientLight]
	fcomp [float_minusone]
	fstsw ax
	test ah,40h
	     jnz .noskyinit

		mov edi,RenderTempObj+4
		xor eax,eax
		mov ecx,14
		rep stosd
		mov ecx,RenderTempObj
		mov eax,[NIGHTAmbientLight]
		mov DWORD [ecx+8],1				; set only ambient light
		mov [ecx+0Ch],eax

		call [AT06RENMAINPROCADDR]

	.noskyinit:

;-------------
;FOG INIT
;-------------

	mov edi,FogTempObj+4
	xor eax,eax
	mov ecx,5
	rep stosd
	mov ecx,FogTempObj

; fog color

	cmp [AT05FogWithColorModRunOnceInLevel],TRUE		; The following code is only necessary if AT05 was not run upon CStartTrigger (maybe error)
	    jz .nofogcolor

		mov eax,[DefaultFogColorValue]
		cmp [NIGHTEnabled],FALSE
		    jz @f

			mov eax,[NIGHTFogColorCurrent]

		@@:

		or DWORD [ecx+8],CSETFOG_USE_RGB		; this bit tells AT05 to only change fog color
		mov [ecx+0Ch],eax				; new fog colors (based on previous)

	.nofogcolor:

; fog power and fog half for Night Mode

	test [NIGHTEnabled],1
	     jz .nofogpowerhalf

		mov eax,[NIGHTFogPower]
		mov edx,[NIGHTFogHalf]

		cmp eax,-1.00000
		    jz @f

			or DWORD [ecx+8],CSETFOG_USE_FOGPOWER
			mov [ecx+CSetFogAction.FogPower],eax

		@@:

		cmp edx,-1.00000
		    jz @f

			or DWORD [ecx+8],CSETFOG_USE_FOGHALF
			mov [ecx+CSetFogAction.FogHalf],edx

		@@:

	.nofogpowerhalf:

;fog power and fog half for DisableGameFog

	test [DisableGameFog],1
	     jz @f

		or DWORD [ecx+8],CSETFOG_USE_FOGPOWER+CSETFOG_USE_FOGHALF
		mov eax,[FogSettingsForcedFogPowerValue]
		mov [ecx+CSetFogAction.FogPower],eax
		mov eax,[FogSettingsForcedFogHalfValue]
		mov [ecx+CSetFogAction.FogHalf],eax

	@@:

	cmp DWORD [ecx+8],0
	    jz @f						; only call if we're actually changing something

		call [AT05FOGMAINPROCADDR]

	@@:


pop edi
pop esi
ret

;========================================================
INTERCEPTPROC AT06RENDMAINPROCSUBSTITUTE			; ecx = AT06 object
;========================================================

	push ecx
	push esi
	push edi
	lea esi,[ecx+4]
	mov edi,RenderTempObj+4
	mov ecx,14
	rep movsd						; copy AT06 object without VTable (simple - no sub-classes)

	test [NIGHTEnabled],1
	mov ecx,RenderTempObj
	     jz .norendersubstitute

		fld [NIGHTAmbientLight]
		fcom [float_minusone]
		fstsw ax
		test ah,40h
		     jnz @f

			fst DWORD [ecx+0Ch]

		@@:
		fstp st0

	.norendersubstitute:

	call [AT06RENMAINPROCADDR]

	pop edi
	pop esi
	pop ecx

	mov edx,[RenderTempObj+4]				; save reference count in real object (don't know if necessary)
	mov [ecx+4],edx

ret




