;============================================================
; MAIN CPARTICLES COLOR MODIFICATION CODE
;============================================================

;============================================================
ATXPROC CParticlesEndLevelLoad
;============================================================
; most other CParticles-related values are set in PRELEVELLOAD.inc

	cmp [NewGRFLoadDetected],FALSE
	    jz @f

		call CParticlesSaveGameDefaultColors		; MUST SAVE COLORS FOR NEW GRF LOADS ONLY
								; Will save default colors and possible changes
								; made by CParticles class script (to 6 default types only)
	@@:

	call CParticlesCopyInitColorsToCurrent
	call CParticlesApplyAT109BufferToCurrentColors
	call CParticlesColorGlobalUpdate			; will apply Night Mode effect to default particle
								; colors as well as all default particle colors
								; supported and set by AT109
ret


; Note: The following functions are not necessarily currently used.
;============================================================
ATXPROC CParticlesSaveGameDefaultColors 		; (for 6 default effects only)
;============================================================
push esi
push edi

	mov esi,CPTypeIDTable
	mov edi,CPInitialColor

	.saveloop:
	stdcall [CParticlesGetObjProc],[esi],0
	add esp,8
	mov edx,[eax+4] 				; particle type's color
	add esi,4
	mov [edi],edx
	add edi,4
	cmp esi,(CPTypeIDTable+sizeof.CParticlesDefault)
	    jb .saveloop

pop edi
pop esi
ret

;============================================================
ATXPROC CParticlesCopyInitColorsToCurrent
;============================================================
push edi
push esi

	mov esi,CPInitialColor
	mov edi,CPCurrentColor
	mov ecx,(sizeof.CParticlesDefault/4)
	rep movsd

pop esi
pop edi
ret

;============================================================
ATXPROC CParticlesColorGlobalUpdate
;============================================================
; uses CPCurrentColor and CPCurrentColorSub structs to set new color values

push esi
push edi
push ebx

	 mov esi,CPCurrentColor
	 mov edi,CPTypeIDTable
	 mov ebx,CPCurrentColorMax

	 .updateloop:
	 stdcall SpanResize24BitColor,[esi],[ebx]	; apply NIGHT mode color change
	 push eax
	 stdcall [CParticlesGetObjProc],[edi],0 	; get CParticles object
	 add esp,8
	 add esi,4
	 pop ecx
	 add ebx,4
	 add edi,4
	 cmp esi,(CPCurrentColor+sizeof.CParticlesDefault)
	 mov [eax+4],ecx				; update this particle type's color
	     jb .updateloop

pop ebx
pop edi
pop esi
ret

;============================================================
ATXPROC CParticlesSetUniMaskSelective			; sets new color masks for designated particle types
;============================================================
; edx = flags that designate which particle color mask to set
; eax = color sub (24-bit 00RRGGBBh)

push edi

	 mov edi,CPCurrentColorMax
	 mov ecx,(sizeof.CParticlesDefault/4)

	 .setloop:
	 test dl,1
	      jz @f

		mov [edi],eax

	 @@:
	 shr edx,1
	 add edi,4
	 loop .setloop

pop edi
ret

;============================================================
ATXPROC CParticlesResetMaxColorsBuffer
;============================================================
push edi

	 mov edi,CPCurrentColorMax
	 mov eax,00FFFFFFh
	 mov ecx,(sizeof.CParticlesDefault/4)
	 rep stosd

pop edi
ret

;============================================================
ATXPROC CParticlesResetAT109Buffer
;============================================================
push edi

	 mov edi,CPAT109Color
	 mov eax,0FF000000h				; (no-color indicator)
	 mov ecx,(sizeof.CParticlesDefault/4)
	 rep stosd

pop edi
ret

;============================================================
ATXPROC CParticlesApplyAT109BufferToCurrentColors
;============================================================
push esi
push edi

	 mov esi,CPAT109Color
	 mov edi,CPCurrentColor
	 mov ecx,(sizeof.CParticlesDefault/4)
	 mov edx,0FF000000h				; (no-color indicator)

	 .applyloop:
	 mov eax,[esi]

	 cmp eax,edx
	     jz @f

		mov [edi],eax

	 @@:

	 add esi,4
	 add edi,4
	 loop .applyloop

pop edi
pop esi
ret

;============================================================
ATXPROC CParticlesInitTypeIDTable
;============================================================

	 ccall [CParticlesEncodeNameProc],CPSTR_BLOOD
	 mov [CPTypeIDTable.Blood],eax

	 ccall [CParticlesEncodeNameProc],CPSTR_BLEED
	 mov [CPTypeIDTable.Bleed],eax

	 ccall [CParticlesEncodeNameProc],CPSTR_WATER
	 mov [CPTypeIDTable.Water],eax

	 ccall [CParticlesEncodeNameProc],CPSTR_TERRAIN
	 mov [CPTypeIDTable.Terrain],eax

	 ccall [CParticlesEncodeNameProc],CPSTR_TERRAINSAND
	 mov [CPTypeIDTable.TerrainSand],eax

	 ccall [CParticlesEncodeNameProc],CPSTR_METALHARD1
	 mov [CPTypeIDTable.MetalHard1],eax

ret

;============================================================
ATXPROC CParticlesFindTypeIDInTable			; edx = type ID
;============================================================
; returns index into default buffers (no base), or -1 if not found

	 mov ecx,(sizeof.CParticlesDefault/4)
	 mov eax,CPTypeIDTable

	 .findloop:
	 cmp edx,[eax]
	     jz .found
	 add eax,4
	 loop .findloop

	 mov eax,-1

ret

.found:

	 sub eax,CPTypeIDTable

ret


;=========================================================
INTERCEPTPROC CPBASECLIPFROMDISPLAYINTERCEPT
;=========================================================
; used for CParticleFluid only

; ecx = 1000.0
; edi = CParticleBase instance
; must return al = false (no particle display this round)

	cmp [CPBaseClipFromDispEnabled],0
	    jnz @f

		mov [edi+4],ecx

	@@:

	xor al,al

pop edi
pop esi
add esp,40h
ret 8

;=========================================================
INTERCEPTPROC CPBASECLIPFROMDISPLAYSOLIDINTERCEPT
;=========================================================
; used for CParticleSolid and CParticleDust

; ecx = 1000.0
; edi = CParticleBase instance
; must return al = false (no particle display this round)

	cmp [CPBaseClipFromDispEnabled],0
	    jnz @f

		mov [edi+4],ecx

	@@:

	xor al,al

pop edi
pop esi
pop ebp
pop ebx
add esp,24h
ret 8










