struct LIST
       lpFirstNode	dd	?
       lpLastNode	dd	?
       numNodes 	dd	?
       heapHandle	dd	?
ends

struct LISTNODE
       lpNextNode	dd	?
       lpPrevNode	dd	?
       value		dd	?
ends


;=================================================================
freeList:	; ecx = LIST instance
;=================================================================
push esi
push edi
push ebx
mov esi,ecx

	mov edi,[esi+LIST.lpFirstNode]
	mov ebx,[esi+LIST.heapHandle]
	test edi,edi
	     jz .listEmpty
	.freeLoop:
	mov ecx,edi
	mov edi,[edi+LISTNODE.lpNextNode]
	invoke HeapFree,ebx,0,ecx
	test edi,edi
	     jnz .freeLoop

	xor eax,eax
	mov [esi+LIST.lpFirstNode],eax
	mov [esi+LIST.lpLastNode],eax
	mov [esi+LIST.numNodes],eax

.listEmpty:

pop ebx
pop edi
pop esi
ret

;=================================================================
addLastToList:	     ; ecx = LIST instance, eax = value to add
;=================================================================
; returns NULL if error or node address otherwise
push esi
push edi
mov esi,ecx

	mov edi,[esi+LIST.lpLastNode]
	test edi,edi
	     jz .firstNode
	push eax
	invoke HeapAlloc,[esi+LIST.heapHandle],0,sizeof.LISTNODE
	test eax,eax
	pop edx
	     jz .exit

	mov [eax+LISTNODE.lpNextNode],0
	mov [eax+LISTNODE.lpPrevNode],edi
	mov [eax+LISTNODE.value],edx
	mov [edi+LISTNODE.lpNextNode],eax
	mov [esi+LIST.lpLastNode],eax
	inc [esi+LIST.numNodes]
.exit:
pop edi
pop esi
ret

.firstNode: ; eax = value
	mov ecx,esi
	call addFirstNode
pop edi
pop esi
ret

;=================================================================
removeLastFromList:	  ; ecx = LIST instance
;=================================================================
; returns value of removed node in eax
push esi
push edi
mov esi,ecx

	mov edx,[esi+LIST.lpLastNode]
	mov edi,[edx+LISTNODE.lpPrevNode]
	push [edx+LISTNODE.value]
	invoke HeapFree,[esi+LIST.heapHandle],0,edx
	test edi,edi
	pop eax
	     jz .wasFirstNode

	mov [edi+LISTNODE.lpNextNode],0
	mov [esi+LIST.lpLastNode],edi
	dec [esi+LIST.numNodes]

pop edi
pop esi
ret

.wasFirstNode:
	xor edx,edx
	mov [esi+LIST.lpFirstNode],edx
	mov [esi+LIST.lpLastNode],edx
	mov [esi+LIST.numNodes],edx

pop edi
pop esi
ret



;=================================================================
addFirstNode:	       ; ecx = LIST instance, eax = value to add
;=================================================================
; returns NULL if error or address of new node if OK

	push eax
	push ecx
	invoke HeapAlloc,[ecx+LIST.heapHandle],0,sizeof.LISTNODE
	test eax,eax
	pop ecx
	pop edx
	    jz .exit

	mov [ecx+LIST.lpFirstNode],eax
	mov [ecx+LIST.lpLastNode],eax
	mov [ecx+LIST.numNodes],1

	mov [eax+LISTNODE.lpNextNode],0
	mov [eax+LISTNODE.lpPrevNode],0
	mov [eax+LISTNODE.value],edx

.exit:
ret


